/*
bhnetwork.h
Copyright (C) 2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY. 
*/
#ifndef _CNETWORK_
#define _CNETWORK_
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <net/if.h>
#if defined(FREEBSD) || defined(NETBSD) || defined(OPENBSD)
#include <sys/linux_ioctl.h> 
#endif 
#include <sigc++/object.h>
#include <sigc++/class_slot.h>
#include <gtkmm/main.h>

/**
 * \defgroup bhnetwork Network
 * \@{
 *
 * \ingroup mflib
 * \@{	
 */
/*! \brief A class to handle the network. */
class CNetwork : public sigc::trackable
{
private:
	bool          bTcp;
	ip_mreq       mreq;
	sockaddr_in   sin;
	sockaddr_in   csin;
	sockaddr_in   posserv;
	sockaddr_in   ctrlserv;
   char          devname[16];
	unsigned char macaddr[6];	
	
	bool bHaveServerAddress;
	bool bHaveCtrlAddress;
   bool bHaveDevice;
	
	bool  bBackoff;
	float backoff_value;
	int   retries;
	int   max_retries;
	bool  bResend;
	bool  bTimeout;
	gint  timer_tag;
	gint  sd_tag;
	gint  csd_tag;
	int   timer_value;
	
	int		 status, err;
	socklen_t size;
	timeval	 to;
	

protected:
	BhPosHdr   	txhdr;
	BhPosHdr   	rxhdr;
	BhMcPosMsg	ctrlhdr;
   Glib::Mutex netmutex;
   Glib::Mutex ctrlmutex;
   
   //Glib::RefPtr<IOSource> net;

   sigc::signal<void, int, void *> msg_signal;
   sigc::signal<void, int, void *> ctrlmsg_signal;
   sigc::signal<void> timeout_signal;

public:
	int	sd, ctrl_sd;

	CNetwork();
   CNetwork(Gtk::Main *);
	~CNetwork();
	int   Open();
	void  Close();
	int   GetMac(unsigned char *);
	int   GetMsgID();
	void  SetMsgID(int);
	int   Send(BhPosHdr *);
	int   ReSend();
	
	int	SetServer(int family, char *address, unsigned short port);
	int	SetCtrl(int family, char *address, unsigned short port);
   int   SetDevice(char *dev);
	
	gint Timeout();
	void Activity(int d, GdkInputCondition c);
	void Ctrl_Activity(int d, GdkInputCondition c);
	int  Send(BhPosHdr *hdr, bool bRs, bool bTo, int to_val);

   sigc::signal<void, int, void *> signal_msg(){ return msg_signal; }
   sigc::signal<void, int, void *> signal_ctrlmsg(){ return ctrlmsg_signal; }
   sigc::signal<void> signal_timeout(){ return timeout_signal; }
};
/* @} 
	@} */
#endif // _CNETWORK_
