/*
bhtranslist.h
Copyright (C) 2004  Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY. 
*/
#ifndef _BHITEMLIST_H
#define _BHITEMLIST_H

#include <gdkmm/colormap.h>
#include <gtkmm/liststore.h>
#include <gtkmm/treeview.h>
#include <gtkmm/treeviewcolumn.h>
#include <gtkmm/cellrenderertext.h>
#include <gtkmm/cellrendererpixbuf.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/entry.h>
#include <gtkmm/eventbox.h>
#include <gtkmm/label.h>
#include <gtkmm/menu.h>
#include "bhcellrenderer.h"

#ifndef _BHHDR_H_
#include <pos.h>
#endif
#ifndef _CTRANSITEM_
#include <ctransitems.h>
#endif
#ifndef _CCASH_H_
#include <cash.h>
#endif

namespace bhpos{

/**
 * \ingroup mflib
 * @{	
 */
   
namespace widget{

/**
 * \defgroup bhtranslist Transaction List
 * @{
 *
 * \ingroup bhwidget
 * @{	
 */

enum BhCompletionType{
   BHITEMLIST_COMPLETION_PLU,
   BHITEMLIST_COMPLETION_QTY,
   BHITEMLIST_COMPLETION_DESCEDIT,
   BHITEMLIST_COMPLETION_PRICEEDIT,
   BHITEMLIST_COMPLETION_INCEDIT,
   BHITEMLIST_COMPLETION_DECEDIT,
   BHITEMLIST_COMPLETION_REMOVE,
   BHITEMLIST_COMPLETION_CASHITEM,
   BHITEMLIST_COMPLETION_PAYAMOUNT,
   BHITEMLIST_COMPLETION_PAYBALANCE,
   BHITEMLIST_COMPLETION_PAYCHANGE,
   BHITEMLIST_COMPLETION_PAYAMOUNTMORE,
   BHITEMLIST_COMPLETION_PAYERROR,
   BHITEMLIST_COMPLETION_TENDER,
   BHITEMLIST_COMPLETION_PLUCANCELLED,
   BHITEMLIST_COMPLETION_QTYCANCELLED,
   BHITEMLIST_COMPLETION_DESCEDITCANCELLED,
   BHITEMLIST_COMPLETION_PRICEEDITCANCELLED,
   BHITEMLIST_COMPLETION_CASHITEMCANCELLED,
   BHITEMLIST_COMPLETION_PAYAMOUNTCANCELLED,
   BHITEMLIST_COMPLETION_PAYBALANCECANCELLED,
   BHITEMLIST_COMPLETION_TENDERCANCELLED,
};

enum BhItemListOption{
   BHITEMLIST_OPTION_TAXINCLUSIVE,
   BHITEMLIST_OPTION_PAYTYPES
   /*
   BHITEMLIST_OPTION_DEFAULT=0,
   BHITEMLIST_OPTION_EXPAND_TREE=1,
   BHITEMLIST_OPTION_ENABLE_MOUSE=2,
   BHITEMLIST_OPTION_ENABLE_POPUP=4*/
};

enum BhNavigationType{
   BHNAV_TYPE_UP,
   BHNAV_TYPE_DOWN,
   BHNAV_TYPE_PGUP,
   BHNAV_TYPE_PGDOWN,
   BHNAV_TYPE_HOME,
   BHNAV_TYPE_END,
   BHNAV_TYPE_LEFT,
   BHNAV_TYPE_RIGHT
};

/*! TransColumns
 \brief A class to handle item columns
*/
class TransColumns : public Gtk::TreeModel::ColumnRecord
{
public:
   TransColumns();
   Gtk::TreeModelColumn<Glib::ustring> status;
   Gtk::TreeModelColumn<Glib::ustring> station;
   Gtk::TreeModelColumn<Glib::ustring> description;
   Gtk::TreeModelColumn<Glib::ustring> total;
   Gtk::TreeModelColumn<int>           type;	      // hidden col
   Gtk::TreeModelColumn<Gdk::Color>    background; // hidden col
   Gtk::TreeModelColumn<Gdk::Color>    foreground; // hidden col
};


/** bhtranslist class 
* \brief GUI implementation of the gkmm treeview widget.
* A trans items list.
*
* This version is also mouse enabled, unlike previous versions, but this 
* can be disabled by setting the option. The reason for this was touchscreen
* interfacing.
*/ 
class BhTransactionList : public Gtk::TreeView
{
private:
   Glib::ustring buffer;
   Gtk::TreeModel::iterator curr_row;
   int           i_curr_row;
   int           row_height;
   int           num_rows;
   int           curr_column;
   bool          bHaveRow;

   Gtk::CellRendererText *cell_col0;
   Gtk::CellRendererText  *cell_col1;
   Gtk::CellRendererText  *cell_col2;
   Gtk::CellRendererText  *cell_col3;
   
   bool IsValidPath(Gtk::TreeModel::Path path);
   
   bool is_last_row(Gtk::TreeModel::iterator& iter_);
   void scroll(bool bUp);

protected:
	TransColumns                     columns;
   Glib::RefPtr<Gtk::ListStore>     refls;
   Glib::RefPtr<Gtk::TreeSelection> refSelection;
   Gtk::ScrolledWindow              *scroll_win;
   
   void SetRowColour(Gtk::TreeModel::iterator iter);
   void SetRowColour(Gtk::TreeModel::Row& row);
   

   //int TransListMode(BhTransList *tl, bool bShow);
   sigc::signal<void, int, Glib::ustring&> pluentry_signal;
   sigc::signal<void, BhCompletionType, void *, void *> completion_signal;

public:
   BhTransactionList(Gtk::ScrolledWindow *sw);
   ~BhTransactionList();
   void SetOption(BhItemListOption, void *);
   void Resize();
   /** Load a transaction 
    *  <tt>Clears current items, loads new transaction</tt>
    *  \param transaction to be filled.
    *  \return 1 is trans has 0 items, or bad load, 1 otherwise.
    */
   int  Load(CTransItems& transaction);
   int  TransList(BhTransList *tl, bool bShow);
   int  TransList(BhSSTransList *tl, bool bShow);
   void Clear();
   
   /** General Action
    *  \brief A general action to perform
    *  \param type Action type.
    *  \return true if action failed, false otherwise.
    */
   bool Navigation(BhNavigationType n);
   
   sigc::signal<void, int, Glib::ustring&> signal_pluentry(){ return pluentry_signal; }
   sigc::signal<void, BhCompletionType, void *, void *> signal_completion(){ return completion_signal; }
};

/* @} 
	@} */

}

/* @} */

}

#endif   // _BHTRANSLIST_H
